#!/usr/bin/python
# -*- coding: utf-8 -*-

# this script checks include guards of c, cpp - header files
# and removes them:
# e.g.: test.h
# #if !defined(foo)
# #define foo ...
# #endif // foo
#
# parser works with following characteristics:
#  - the include guard doesn't have to start at the first line
#    and doesn't have to finish at the last line
#  - only one guard per file
#  - deleting first include-guard of following syntax
#  - include is formated like:
#     line x  : #if !defined(foo)
#     line x+1: #define foo
#     line x+2:  ...
#     line x+y: #endif

import os, re, sys

__debug = False

def apply(fname):
	fname = os.path.abspath(fname)
	fd = open(fname, 'r')

	# check for include-guard
	rg = re.compile('#if\s+!defined\((\S+)\)')
	rd = re.compile('#define\s+(\S+)')

	# status variables for parsing
	ifcount = 0
	foundfirstifdef = False
	foundfirstdefine = False
	guardbegindeleted = False
	guardenddeleted = False
	guardifdefline = ''
	guarddefineline = ''
	guardname  = ''
	guarddef = ''

	for line in fd:
		line = line.strip()

		# #if
		if line.startswith('#if'):
			ifcount += 1

		if foundfirstifdef == False and guardbegindeleted == False and guardname == '':
			# #if !defined(guardname)
			foundfirstifdef = True
			regres = rg.match(line)
			if (regres):
				guardifdefline = line
				guardname = regres.groups()[0]
			else: print(line)
			continue

		if foundfirstdefine == False and guardbegindeleted == False and guardname != '':
			# #define guardname
			foundfirstdefine = True
			regres   = rd.match(line)
			if (regres):
				guarddef = regres.groups()[0]
				if (guarddef != guardname):
					print(guardifdefline)
					print(line)
				else: guardbegindeleted = True
			else:
				print(guardifdefline)
				print(line)
				guardifdefline = ''
			continue

		# #endif
		if line.startswith('#endif'):
			if (ifcount == 1) and guardbegindeleted and guardenddeleted == False:
				guardenddeleted = True
			else:
				print(line.strip())
			ifcount -= 1
			continue

		# dump other lines
		print(line.strip())

	fd.close()
	if guardbegindeleted != guardenddeleted:
		print("ERROR: include guard deletion failed!")


def usage():
	print(sys.argv[0] + ' filename')
	print('programm writes results to stdout')


##################################################
if __name__ == '__main__':
	if len(sys.argv) < 2:
		usage()
	else:
		apply(sys.argv[1])
